package com.fjhx.common.service.logistics.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.common.entity.logistics.dto.LogisticsEndingAmountSelectDto;
import com.fjhx.common.entity.logistics.po.LogisticsAmount;
import com.fjhx.common.entity.logistics.po.LogisticsCompanyInfo;
import com.fjhx.common.entity.logistics.po.LogisticsEndingAmount;
import com.fjhx.common.entity.logistics.vo.LogisticsEndingAmountVo;
import com.fjhx.common.mapper.logistics.LogisticsEndingAmountMapper;
import com.fjhx.common.service.logistics.LogisticsAmountService;
import com.fjhx.common.service.logistics.LogisticsCompanyInfoService;
import com.fjhx.common.service.logistics.LogisticsEndingAmountService;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 物流期末余额 服务实现类
 * </p>
 *
 * @author
 * @since 2024-05-06
 */
@Service
public class LogisticsEndingAmountServiceImpl extends ServiceImpl<LogisticsEndingAmountMapper, LogisticsEndingAmount> implements LogisticsEndingAmountService {

    @Autowired
    private LogisticsCompanyInfoService logisticsCompanyInfoService;
    @Autowired
    private LogisticsAmountService logisticsAmountService;

    /**
     * 每月1日0点1分 赋值上月期末余额
     */
//    @PostConstruct
    @Scheduled(cron = "0 1 0 1 * ?")
    public void autoSaveEndingAmount() {
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);
        SecurityUtils.setTenantId("000000");
        DateTime lastMonth = DateUtil.offsetMonth(new Date(), -1);
        int year = DateUtil.year(lastMonth);
        int month = DateUtil.month(lastMonth) + 1;

        LogisticsEndingAmountSelectDto dto = new LogisticsEndingAmountSelectDto();
        dto.setYear(year);

        List<JSONObject> jsonObjects = logisticsRechargeSummary(dto);
        for (JSONObject json : jsonObjects) {
            Long logisticsCompanyId = json.getLong("id");
            LogisticsEndingAmountVo logisticsEndingAmountVo = json.getObject("" + month, LogisticsEndingAmountVo.class);

            //更新期末余额
            String format = StrUtil.format("{}-{}", year, month);
            LogisticsEndingAmount one = this.getOne(q -> q
                    .eq(LogisticsEndingAmount::getLogisticsCompanyId, logisticsCompanyId)
                    .apply("DATE_FORMAT(account_date,'%Y-%m') = DATE_FORMAT( {0}, '%Y-%m' )", DateUtil.parse(format, "yyyy-MM"))
            );
            if (ObjectUtil.isEmpty(one)) {
                one = new LogisticsEndingAmount();
                one.setLogisticsCompanyId(logisticsCompanyId);
                one.setAccountDate(DateUtil.parse(format, "yyyy-MM"));
                one.setEndingBalance(BigDecimal.ZERO);
            }

            //赋值期末余额
            BigDecimal endingBalance = logisticsEndingAmountVo.getEndingBalance();
            if (ObjectUtil.isEmpty(endingBalance)) {
                endingBalance = BigDecimal.ZERO;
            }
            one.setEndingBalance(endingBalance);

            //期末应付余额
            BigDecimal endingPayableBalance = logisticsEndingAmountVo.getEndingPayableBalance();
            if (ObjectUtil.isEmpty(endingPayableBalance)) {
                endingPayableBalance = BigDecimal.ZERO;
            }
            one.setEndingPayableBalance(endingPayableBalance);


            this.saveOrUpdate(one);
        }
        SecurityUtils.clearTenantId();
        DynamicDataSourceContextHolder.clear();
    }

    @Override
    public List<JSONObject> logisticsRechargeSummary(LogisticsEndingAmountSelectDto dto) {
        List<JSONObject> outList = new ArrayList<>();

        //默认赋值年
        if (ObjectUtil.isEmpty(dto.getYear())) {
            dto.setYear(DateUtil.year(new Date()));
        }

        //费控 预付、付款 信息
        List<LogisticsEndingAmountVo> logisticsRechargeSummaryBos = baseMapper.logisticsRechargeSummary(dto);
        Map<Long, List<LogisticsEndingAmountVo>> logisticsRechargeMap = logisticsRechargeSummaryBos.stream()
                .collect(Collectors.groupingBy(LogisticsEndingAmountVo::getLogisticsCompanyId));

        //快递公司 每月发生额
        Map<Long, LogisticsAmount> logisticsAmountMap = logisticsAmountService.mapKEntity(LogisticsAmount::getLogisticsCompanyId, q -> q.eq(LogisticsAmount::getYear, dto.getYear()));

        //遍历物流公司
        List<LogisticsCompanyInfo> list = logisticsCompanyInfoService.list();
        for (LogisticsCompanyInfo logisticsCompanyInfo : list) {
            JSONObject logisticsCompanyInfoJson = JSONObject.parseObject(JSONObject.toJSONString(logisticsCompanyInfo));
            //赋值年初余额,年初应付
            BigDecimal yearBeginBalance = BigDecimal.ZERO;
            BigDecimal yearBeginPayableBalance = BigDecimal.ZERO;

            String lastYearEndMonth = StrUtil.format("{}-12", (dto.getYear() - 1));
            LogisticsEndingAmount lastYearEndAmount = this.getOne(q -> q
                    .eq(LogisticsEndingAmount::getLogisticsCompanyId, logisticsCompanyInfo.getId())
                    .apply("DATE_FORMAT(account_date,'%Y-%m') = DATE_FORMAT( {0}, '%Y-%m' )", DateUtil.parse(lastYearEndMonth, "yyyy-MM"))
            );
            if (ObjectUtil.isNotEmpty(lastYearEndAmount)) {
                yearBeginBalance = lastYearEndAmount.getEndingBalance();
                yearBeginPayableBalance = lastYearEndAmount.getEndingPayableBalance();
            }
            //排空
            yearBeginBalance = ObjectUtil.defaultIfNull(yearBeginBalance, BigDecimal.ZERO);
            yearBeginPayableBalance = ObjectUtil.defaultIfNull(yearBeginPayableBalance, BigDecimal.ZERO);

            logisticsCompanyInfoJson.put("yearBeginBalance", yearBeginBalance);
            logisticsCompanyInfoJson.put("yearBeginPayableBalance", yearBeginPayableBalance);

            //上一期末余额
            BigDecimal lastEndingBalance = yearBeginBalance;
            //上一期末应付余额
            BigDecimal lastEndingPayableBalance = yearBeginPayableBalance;

            //赋值每一个月信息
            List<LogisticsEndingAmountVo> list1 = logisticsRechargeMap.getOrDefault(logisticsCompanyInfo.getId(), new ArrayList<>());
            Map<Integer, LogisticsEndingAmountVo> collect = list1.stream().collect(Collectors.toMap(LogisticsEndingAmountVo::getAccountPeriodMonth, Function.identity()));
            for (int i = 1; i <= 12; i++) {
                LogisticsEndingAmountVo logisticsRecharge = collect.getOrDefault(i, new LogisticsEndingAmountVo());

                //预付
                BigDecimal advanceAmount = logisticsRecharge.getAdvanceAmount();
                if (advanceAmount == null) {
                    advanceAmount = BigDecimal.ZERO;
                } else {
                    //数据转负数
                    advanceAmount = BigDecimal.ZERO.subtract(advanceAmount);
                    logisticsRecharge.setAdvanceAmount(advanceAmount);
                }

                //应付报表抵扣（负数）
                BigDecimal paymentDeductionAmount = BigDecimal.ZERO;
                //抵扣
                BigDecimal deductionAmount = logisticsRecharge.getDeductionAmount();
                if (deductionAmount == null) {
                    deductionAmount = BigDecimal.ZERO;
                } else {
                    //数据转负数
                    paymentDeductionAmount = BigDecimal.ZERO.subtract(deductionAmount);
                    logisticsRecharge.setPaymentDeductionAmount(paymentDeductionAmount);
                }
                //付款
                BigDecimal paymentAmount = logisticsRecharge.getPaymentAmount();
                if (paymentAmount == null) {
                    paymentAmount = BigDecimal.ZERO;
                } else {
                    //数据转负数
                    paymentAmount = BigDecimal.ZERO.subtract(paymentAmount);
                    logisticsRecharge.setPaymentAmount(paymentAmount);
                }

                //期末余额 = 上期末余额 + 抵扣（贷方）+ 预付（借方）负数
                logisticsRecharge.setEndingBalance(lastEndingBalance.add(deductionAmount).add(advanceAmount));

                //获取 贷方发生额
                LogisticsAmount logisticsAmount = logisticsAmountMap.getOrDefault(logisticsCompanyInfo.getId(), new LogisticsAmount());
                JSONObject logisticsAmountJson = JSONObject.parseObject(JSONObject.toJSONString(logisticsAmount));
                BigDecimal creditAmount = logisticsAmountJson.getBigDecimal(StrUtil.format("month{}Amount", i));
                if (creditAmount == null) {
                    creditAmount = BigDecimal.ZERO;
                }
                logisticsRecharge.setCreditAmount(creditAmount);

                //期末应付余额 = 上期末应付余额 + 贷方发生额 + 借方付款(负数) + 抵扣(负数)
                BigDecimal endingPayableBalance = lastEndingPayableBalance.add(creditAmount).add(paymentAmount).add(paymentDeductionAmount);
                logisticsRecharge.setEndingPayableBalance(endingPayableBalance);

                logisticsCompanyInfoJson.put("" + i, logisticsRecharge);

                //赋值上一期末余额，上一期末应付余额
                lastEndingBalance = logisticsRecharge.getEndingBalance();
                lastEndingPayableBalance = logisticsRecharge.getEndingPayableBalance();
            }

            outList.add(logisticsCompanyInfoJson);
        }

        return outList;
    }

    @Override
    public List<JSONObject> logisticsPayableSummary(LogisticsEndingAmountSelectDto dto) {
        return logisticsRechargeSummary(dto);
    }
}
