package com.fjhx.form.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fjhx.common.utils.excel.util.ExcelUtil;
import com.fjhx.form.entity.*;
import com.fjhx.form.mapper.ReportMapper;
import com.fjhx.form.service.ReportService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.report.po.ReportLossesDetails;
import com.fjhx.mes.service.report.ReportLossesDetailsService;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.common.utils.wrapper.SqlField;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ReportServiceImpl implements ReportService {

    private final ReportMapper reportMapper;
    private final ReportLossesDetailsService reportLossesDetailsService;
    private final ProductInfoService productInfoService;
    private final ISysDeptService deptService;

    @Autowired
    public ReportServiceImpl(ReportMapper reportMapper, ReportLossesDetailsService reportLossesDetailsService, ProductInfoService productInfoService, ISysDeptService sysDeptService) {
        this.reportMapper = reportMapper;
        this.reportLossesDetailsService = reportLossesDetailsService;
        this.productInfoService = productInfoService;
        this.deptService = sysDeptService;
    }

    @Override
    public Page<ProductionReportBo> productionReport(ProductionReportSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();

        Page<ProductionReportBo> page = reportMapper.productionReport(dto.getPage(), wrapper);
        List<ProductionReportBo> productionReportBos = page.getRecords();
        List<Long> ptIds = productionReportBos.stream().map(ProductionReportBo::getId).collect(Collectors.toList());


        //获取部门列表
        List<SysDept> companyList = deptService.list();
        Map<Long, String> companyMap = companyList.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        //获取报损信息
        List<ReportLossesDetails> reportLossesDetails = reportLossesDetailsService.list(q -> q.in(ReportLossesDetails::getProdTaskId, ptIds));
        Map<Long, List<ReportLossesDetails>> reportLossesMap = reportLossesDetails.stream().collect(Collectors.groupingBy(ReportLossesDetails::getProdTaskId));

        for (ProductionReportBo vo : productionReportBos) {
            vo.setFactoryName(companyMap.get(vo.getFactoryId()));

            List<ReportLossesDetails> reportLossesList = reportLossesMap.getOrDefault(vo.getId(), new ArrayList<>());
            //原材料损耗 数量，成本
            List<ReportLossesDetails> rawReportLosses = reportLossesList.stream()
                    .filter(item -> ObjectUtil.equals(item.getMaterialType(), 1)).collect(Collectors.toList());
            BigDecimal rawLossCount = rawReportLosses.stream().map(ReportLossesDetails::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal rawLossAmount = rawReportLosses.stream().map(ReportLossesDetails::getMaterialPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
            vo.setRawLossCount(rawLossCount);
            vo.setRawLossAmount(rawLossAmount);
            //普通物料 损耗 数量，成本
            List<ReportLossesDetails> bomReportLosses = reportLossesList.stream()
                    .filter(item -> ObjectUtil.notEqual(item.getMaterialType(), 1)).collect(Collectors.toList());
            BigDecimal bomLossCount = bomReportLosses.stream().map(ReportLossesDetails::getQuantity).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal bomLossAmount = bomReportLosses.stream().map(ReportLossesDetails::getMaterialPrice).reduce(BigDecimal.ZERO, BigDecimal::add);
            vo.setBomLossCount(bomLossCount);
            vo.setBomLossAmount(bomLossAmount);
            //返工 道数，成本
            Integer reworkCount = bomReportLosses.stream().map(ReportLossesDetails::getReCount).reduce(Integer::sum).orElse(0);
            BigDecimal reworkAmount = bomReportLosses.stream().map(ReportLossesDetails::getRePrice).reduce(BigDecimal.ZERO, BigDecimal::add);
            vo.setReworkCount(reworkCount);
            vo.setReworkAmount(reworkAmount);
            //出错率
            BigDecimal multiply = vo.getRawLossCount().divide(vo.getQuantity(), 4, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100));
            vo.setErrRate(multiply);
        }

        return page;
    }

    @Override
    public Page<SaleReportBo> saleReport(ProductionReportSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        Page<SaleReportBo> saleReportBoPage = reportMapper.saleReport(dto.getPage(), wrapper);
        List<SaleReportBo> records = saleReportBoPage.getRecords();

        //获取部门列表
        List<SysDept> deptList = deptService.list();
        Map<Long, String> deptMap = deptList.stream().collect(Collectors.toMap(SysDept::getDeptId, SysDept::getDeptName));

        for (SaleReportBo record : records) {
            //业务公司
            record.setOfCompanyName(deptMap.get(record.getOfCompanyId()));
            //业务部门
            record.setDeptName(deptMap.get(record.getDeptId()));
            //工厂
            record.setCompanyName(deptMap.get(record.getCompanyId()));
            //不及时率
            BigDecimal untimelyRate = record.getOutInterval().divide(BigDecimal.valueOf(30), 2, RoundingMode.HALF_UP).multiply(BigDecimal.valueOf(100));
            record.setUntimelyRate(untimelyRate);
        }
        return saleReportBoPage;
    }

    @Override
    public List<WaitShipmentReportStatisticBo> waitShipmentReportStatistic() {
        List<WaitShipmentReportStatisticBo> records = reportMapper.waitShipmentReportStatistic();

        //赋值业务员
        UserUtil.assignmentNickName(records, WaitShipmentReportStatisticBo::getSalesmanId, WaitShipmentReportStatisticBo::setSalesmanName);

        return records;
    }

    /**
     * 已完工未出货报表
     */
    @Override
    public Page<WaitShipmentReportBo> waitShipmentReport(ProductionReportSelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        Page<WaitShipmentReportBo> waitShipmentReportBoPage = reportMapper.waitShipmentReport(dto.getPage(), wrapper);
        if (ObjectUtil.isEmpty(waitShipmentReportBoPage)) {
            return waitShipmentReportBoPage;
        }
        List<WaitShipmentReportBo> records = waitShipmentReportBoPage.getRecords();
        for (WaitShipmentReportBo record : records) {
            //超期修正
            if (ObjectUtil.isEmpty(record.getOverdueDay()) || record.getOverdueDay() < 0) {
                record.setOverdueDay(0);
            }
        }

        //赋值业务员
        UserUtil.assignmentNickName(records, WaitShipmentReportBo::getSalesmanId, WaitShipmentReportBo::setSalesmanName);

        //赋值产品信息
        productInfoService.attributeAssign(records, WaitShipmentReportBo::getProductId, (item, productInfo) -> {
            item.setProductCode(productInfo.getCustomCode());
            item.setProductName(productInfo.getName());
            item.setProductLength(productInfo.getLength());
            item.setProductWidth(productInfo.getWidth());
            item.setProductHeight(productInfo.getHeight());
            item.setProductColor(productInfo.getColor());
        });

        return waitShipmentReportBoPage;
    }

    /**
     * 订单汇总查询条件
     */
    private void setOrderSummaryWrapper(OrderSummarySelectDto dto, IWrapper<Object> wrapper) {
        if (ObjectUtil.isNotEmpty(dto.getAccountAge())) {
            wrapper.having("accountAge = {0}", dto.getAccountAge());
        }
        if (ObjectUtil.isNotEmpty(dto.getIsSettlement())) {
            wrapper.having("isSettlement = {0}", dto.getIsSettlement());
        }
        wrapper.keyword(dto.getKeyword(),
                new SqlField("c.code"),
                new SqlField("cu.name"),
                new SqlField("cu.short_name")
        );
        wrapper.orderByDesc("c.create_time", "c.id");
    }

    /**
     * 订单汇总数据处理
     */
    private void setOrderSummaryInfo(List<OrderSummaryBo> records) {
        UserUtil.assignmentNickName(records, OrderSummaryBo::getSalesmanId, OrderSummaryBo::setSalesmanName);
        UserUtil.assignmentNickName(records, OrderSummaryBo::getMerchUserId, OrderSummaryBo::setMerchUserName);
    }

    @Override
    public Page<OrderSummaryBo> orderSummary(OrderSummarySelectDto dto) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        //赋值查询条件
        setOrderSummaryWrapper(dto, wrapper);
        Page<OrderSummaryBo> page = reportMapper.orderSummary(dto.getPage(), dto, wrapper);
        List<OrderSummaryBo> records = page.getRecords();
        //赋值信息
        setOrderSummaryInfo(records);
        return page;
    }

    @Override
    public void orderSummaryExcelExport(OrderSummarySelectDto dto, HttpServletResponse httpServletResponse) {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        //赋值查询条件
        setOrderSummaryWrapper(dto, wrapper);
        List<OrderSummaryBo> records = reportMapper.orderSummary(dto, wrapper);
        //赋值信息
        setOrderSummaryInfo(records);

        //Excel数据处理
        for (OrderSummaryBo record : records) {
            //订单类型处理
            String contractType = record.getContractType();
            switch (contractType) {
                case "1":
                    contractType = "外贸订单（退税）";
                    break;
                case "2":
                    contractType = "内销订单";
                    break;
                case "3":
                    contractType = "打样订单";
                    break;
                case "4":
                    contractType = "外贸订单（不退税）";
                    break;
            }
            record.setContractType(contractType);

            //订单性质处理
            String belongType = record.getBelongType();
            switch (belongType) {
                case "1":
                    belongType = "归属工厂";
                    break;
                case "2":
                    belongType = "归属业务";
                    break;
            }
            record.setBelongType(belongType);

            //是否可结算
            String isSettlement = record.getIsSettlement();
            switch (isSettlement) {
                case "0":
                    isSettlement = "否";
                    break;
                case "1":
                    isSettlement = "是";
                    break;
            }
            record.setIsSettlement(isSettlement);

            //是否已结算处理
            String settleStart = DateUtil.format(record.getSettleStartDate(), "yyyy-MM-dd");
            String settleEnd = DateUtil.format(record.getSettleEndDate(), "yyyy-MM-dd");
            record.setSettleDateStr(String.format("%s~%s ", ObjectUtil.isEmpty(settleStart) ? "" : settleStart, ObjectUtil.isEmpty(settleEnd) ? "" : settleEnd));

            //出货日期处理
            record.setFirstTruckDateStr(DateUtil.format(record.getFirstTruckDate(), "yyyy-MM-dd"));
        }

        //导出Excel
        ExcelUtil.export(httpServletResponse, records, OrderSummaryBo.class);
    }
}
